@@ -9,5 +9,4 @@ if [ $# -ne 2 ]; then
   echo "$usage"; exit
 fi
 
-<bindir>/wdiff -1n $1 $2 |
-  sed -e 's/^/  /;/{+/s/^ /|/;s/{+//g;s/+}//g'
+@bindir@/wdiff -1n $1 $2 | sed -e 's/^/  /;/{+/s/^ /|/;s/{+//g;s/+}//g'
