@@ -16,6 +16,10 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+#if HAVE_CONFIG_H
+# include <config.h>
+#endif
+
 /* Exit codes values.  */
 #define EXIT_NO_DIFFERENCES 0	/* no differences found */
 #define EXIT_ANY_DIFFERENCE 1	/* some differences found */
@@ -24,7 +28,7 @@
 /* It is mandatory that some `diff' program is selected for use.  The
    following definition may also include the complete path.  */
 #ifndef DIFF_PROGRAM
-#define DIFF_PROGRAM "diff"
+# define DIFF_PROGRAM "diff"
 #endif
 
 /* One may also, optionnaly, define a default PAGER_PROGRAM.  This might
@@ -37,48 +41,63 @@
 
 /* Library declarations.  */
 
-#ifdef	STDC_HEADERS
-#include <stdlib.h>
+#if STDC_HEADERS
+# include <stdlib.h>
 #endif
 
 #include <ctype.h>
 #include <stdio.h>
 
-#ifdef STDC_HEADERS
-#include <string.h>
-#else /* not STDC_HEADERS */
-#ifdef HAVE_STRING_H
-#include <string.h>
-#else /* not HAVE_STRING_H */
-#include <strings.h>
-#endif /* not HAVE_STRING_H */
-#endif /* not STDC_HEADERS */
+#if STDC_HEADERS
+# include <string.h>
+#else
+# if HAVE_STRING_H
+#  include <string.h>
+# else
+#  include <strings.h>
+#  define strrchr rindex
+# endif
+#endif
 
 char *strstr ();
 
-#ifdef HAVE_TPUTS
-#ifdef HAVE_TERMCAP_H
-#include <termcap.h>
-#else
+#if HAVE_TPUTS
+# if HAVE_TERMCAP_H
+#  include <termcap.h>
+# else
 const char *tgetstr ();
+# endif
 #endif
-#endif /* HAVE_TPUTS */
 
+#include <sys/types.h>
 #include <setjmp.h>
 #include <signal.h>
 #ifndef RETSIGTYPE
-#define RETSIGTYPE void
+# define RETSIGTYPE void
 #endif
 
-#if defined (HAVE_UNISTD_H)
-#include <unistd.h>
+#include <sys/stat.h>
+
+#if HAVE_UNISTD_H
+# include <unistd.h>
+#endif
+
+#include <errno.h>
+#ifndef errno
+extern int errno;
 #endif
 
 #include "getopt.h"
 
+#ifdef PROTOTYPES
+# define _(Args) Args
+#else
+# define _(Args) ()
+#endif
+
 char *getenv ();
-FILE *readpipe ();
-FILE *writepipe ();
+FILE *readpipe _((const char *, ...));
+FILE *writepipe _((const char *, ...));
 char *tmpnam ();
 void error ();
 
@@ -89,14 +108,15 @@ void error ();
 struct option const longopts[] =
 {
   {"copyright"   , 0, NULL, 'C'},
+  {"version"     , 0, NULL, 'V'},
   {"no-deleted"  , 0, NULL, '1'},
   {"no-inserted" , 0, NULL, '2'},
   {"no-common"   , 0, NULL, '3'},
   {"help"        , 0, NULL, 'h'},
+  {"ignore-case" , 0, NULL, 'i'},
   {"printer"     , 0, NULL, 'p'},
   {"statistics"  , 0, NULL, 's'},
   {"terminal"    , 0, NULL, 't'},
-  {"version"     , 0, NULL, 'v'},
   {"start-delete", 1, NULL, 'w'},
   {"end-delete"  , 1, NULL, 'x'},
   {"start-insert", 1, NULL, 'y'},
@@ -104,8 +124,6 @@ struct option const longopts[] =
   {NULL          , 0, NULL, 0}
 };
 
-static char const version[] =
-  "wdiff, version 0.04";
 static char const copyright[] =
   "Copyright (C) 1992 Free Software Foundation, Inc.";
 
@@ -114,6 +132,7 @@ const char *program_name;	/* name of executing program */
 int inhibit_left;		/* inhibit display of left side words */
 int inhibit_right;		/* inhibit display of left side words */
 int inhibit_common;		/* inhibit display of common words */
+int ignore_case;		/* ignore case in comparisons */
 int show_statistics;		/* if printing summary statistics */
 int no_wrapping;		/* end/restart strings at end of lines */
 int autopager;			/* if calling the pager automatically */
@@ -141,13 +160,12 @@ enum copy_mode
 copy_mode;
 
 jmp_buf signal_label;		/* where to jump when signal received */
-int jump_trigger;		/* set when some signal has been received */
+int interrupted;		/* set when some signal has been received */
 
 /* Guarantee some value for L_tmpnam.  */
 #ifndef L_tmpnam
-#include <sys/types.h>
-#include "pathmax.h"
-#define L_tmpnam PATH_MAX
+# include "pathmax.h"
+# define L_tmpnam PATH_MAX
 #endif
 
 typedef struct side SIDE;	/* all variables for one side */
@@ -186,10 +204,10 @@ int count_changed_right;	/* count of changed words in right file */
 | Signal handler.  |
 `-----------------*/
 
-RETSIGTYPE
+static RETSIGTYPE
 signal_handler (int number)
 {
-  jump_trigger = 1;
+  interrupted = 1;
   signal (number, signal_handler);
 }
 
@@ -197,10 +215,10 @@ signal_handler (int number)
 | Prepare to handle signals.  |
 `----------------------------*/
 
-void
+static void
 setup_signals (void)
 {
-  jump_trigger = 0;
+  interrupted = 0;
 
   /* Intercept willingful requests for stopping.  */
 
@@ -212,18 +230,18 @@ setup_signals (void)
 
 /* Terminal initialization.  */
 
-void
+static void
 initialize_strings (void)
 {
-  const char *name;		/* terminal capability name */
-  char term_buffer[2048];	/* terminal description */
-  static char *buffer;		/* buffer for capabilities */
-  char *filler;			/* cursor into allocated strings */
-  int success;			/* tgetent results */
-
-#ifdef HAVE_TPUTS
+#if HAVE_TPUTS
   if (find_termcap)
     {
+      const char *name;		/* terminal capability name */
+      char term_buffer[2048];	/* terminal description */
+      static char *buffer;	/* buffer for capabilities */
+      char *filler;		/* cursor into allocated strings */
+      int success;		/* tgetent results */
+
       name = getenv ("TERM");
       if (name == NULL)
 	error (1, 0, "Specify a terminal type with `setenv TERM <yourtype>'.");
@@ -262,21 +280,25 @@ initialize_strings (void)
 
 /* Character input and output.  */
 
+#if HAVE_TPUTS
+
 /*-----------------------------------------.
 | Write one character for tputs function.  |
 `-----------------------------------------*/
 
-int
+static int
 putc_for_tputs (int chr)
 {
   return putc (chr, output_file);
 }
 
+#endif /* HAVE_TPUTS */
+
 /*---------------------------.
 | Indicate start of delete.  |
 `---------------------------*/
 
-void
+static void
 start_of_delete (void)
 {
 
@@ -286,7 +308,7 @@ start_of_delete (void)
     return;
 
   copy_mode = COPY_DELETED;
-#ifdef HAVE_TPUTS
+#if HAVE_TPUTS
   if (term_delete_start)
     tputs (term_delete_start, 0, putc_for_tputs);
 #endif
@@ -298,7 +320,7 @@ start_of_delete (void)
 | Indicate end of delete.  |
 `-------------------------*/
 
-void
+static void
 end_of_delete (void)
 {
 
@@ -309,7 +331,7 @@ end_of_delete (void)
 
   if (user_delete_end)
     fprintf (output_file, "%s", user_delete_end);
-#ifdef HAVE_TPUTS
+#if HAVE_TPUTS
   if (term_delete_end)
     tputs (term_delete_end, 0, putc_for_tputs);
 #endif
@@ -320,7 +342,7 @@ end_of_delete (void)
 | Indicate start of insert.  |
 `---------------------------*/
 
-void
+static void
 start_of_insert (void)
 {
 
@@ -330,7 +352,7 @@ start_of_insert (void)
     return;
 
   copy_mode = COPY_INSERTED;
-#ifdef HAVE_TPUTS
+#if HAVE_TPUTS
   if (term_insert_start)
     tputs (term_insert_start, 0, putc_for_tputs);
 #endif
@@ -342,7 +364,7 @@ start_of_insert (void)
 | Indicate end of insert.  |
 `-------------------------*/
 
-void
+static void
 end_of_insert (void)
 {
 
@@ -353,7 +375,7 @@ end_of_insert (void)
 
   if (user_insert_end)
     fprintf (output_file, "%s", user_insert_end);
-#ifdef HAVE_TPUTS
+#if HAVE_TPUTS
   if (term_insert_end)
     tputs (term_insert_end, 0, putc_for_tputs);
 #endif
@@ -364,10 +386,10 @@ end_of_insert (void)
 | Skip over white space on SIDE.  |
 `--------------------------------*/
 
-void
+static void
 skip_whitespace (SIDE *side)
 {
-  if (jump_trigger)
+  if (interrupted)
     longjmp (signal_label, 1);
 
   while (isspace (side->character))
@@ -378,10 +400,10 @@ skip_whitespace (SIDE *side)
 | Skip over non white space on SIDE.  |
 `------------------------------------*/
 
-void
+static void
 skip_word (SIDE *side)
 {
-  if (jump_trigger)
+  if (interrupted)
     longjmp (signal_label, 1);
 
   while (side->character != EOF && !isspace (side->character))
@@ -393,10 +415,10 @@ skip_word (SIDE *side)
 | Copy white space from SIDE to FILE.  |
 `-------------------------------------*/
 
-void
+static void
 copy_whitespace (SIDE *side, FILE *file)
 {
-  if (jump_trigger)
+  if (interrupted)
     longjmp (signal_label, 1);
 
   while (isspace (side->character))
@@ -419,12 +441,12 @@ copy_whitespace (SIDE *side, FILE *file)
 	    {
 	      if (no_wrapping && user_delete_end)
 		fprintf (output_file, "%s", user_delete_end);
-#ifdef HAVE_TPUTS
+#if HAVE_TPUTS
 	      if (term_delete_end)
 		tputs (term_delete_end, 0, putc_for_tputs);
 #endif
 	      putc ('\n', output_file);
-#ifdef HAVE_TPUTS
+#if HAVE_TPUTS
 	      if (term_delete_start)
 		tputs (term_delete_start, 0, putc_for_tputs);
 #endif
@@ -446,12 +468,12 @@ copy_whitespace (SIDE *side, FILE *file)
 	    {
 	      if (no_wrapping && user_insert_end)
 		fprintf (output_file, "%s", user_insert_end);
-#ifdef HAVE_TPUTS
+#if HAVE_TPUTS
 	      if (term_insert_end)
 		tputs (term_insert_end, 0, putc_for_tputs);
 #endif
 	      putc ('\n', output_file);
-#ifdef HAVE_TPUTS
+#if HAVE_TPUTS
 	      if (term_insert_start)
 		tputs (term_insert_start, 0, putc_for_tputs);
 #endif
@@ -479,10 +501,10 @@ copy_whitespace (SIDE *side, FILE *file)
 | Copy non white space from SIDE to FILE.  |
 `-----------------------------------------*/
 
-void
+static void
 copy_word (SIDE *side, FILE *file)
 {
-  if (jump_trigger)
+  if (interrupted)
     longjmp (signal_label, 1);
 
   while (side->character != EOF && !isspace (side->character))
@@ -529,17 +551,44 @@ copy_word (SIDE *side, FILE *file)
 | word is on one line.							   |
 `-------------------------------------------------------------------------*/
 
-void
+static void
 split_file_into_words (SIDE *side)
 {
+  struct stat stat_buffer;	/* for checking if file is directory */
 
   /* Open files.  */
 
-  side->file = fopen (side->filename, "r");
-  if (side->file == NULL)
+  if (side->filename == NULL)
     {
-      perror (side->filename);
-      exit (EXIT_OTHER_REASON);
+
+      /* Select a file name, use it for opening a temporary file and
+	 unlink it right away.  Then, copy the whole standard input on
+	 this temporary local file.  Once done, prepare it for reading.
+	 We do not need the file name itself anymore.  */
+
+      tmpnam (side->temp_name);
+      side->file = fopen (side->temp_name, "w+");
+      if (side->file == NULL)
+	error (EXIT_OTHER_REASON, errno, side->temp_name);
+      if (unlink (side->temp_name) != 0)
+	error (EXIT_OTHER_REASON, errno, side->temp_name);
+      while (side->character = getchar (), side->character != EOF)
+	putc (side->character, side->file);
+      rewind (side->file);
+    }
+  else
+    {
+
+      /* Check and diagnose if the file name is a directory.  Or else,
+	 prepare the file for reading.  */
+	 
+      if (stat (side->filename, &stat_buffer) != 0)
+	error (EXIT_OTHER_REASON, errno, side->filename);
+      if ((stat_buffer.st_mode & S_IFMT) == S_IFDIR)
+	error (EXIT_OTHER_REASON, 0, "Directories not supported");
+      side->file = fopen (side->filename, "r");
+      if (side->file == NULL)
+	error (EXIT_OTHER_REASON, errno, side->filename);
     }
   side->character = getc (side->file);
   side->position = 0;
@@ -547,16 +596,13 @@ split_file_into_words (SIDE *side)
   tmpnam (side->temp_name);
   side->temp_file = fopen (side->temp_name, "w");
   if (side->temp_file == NULL)
-    {
-      perror (side->temp_name);
-      exit (EXIT_OTHER_REASON);
-    }
+    error (EXIT_OTHER_REASON, errno, side->temp_name);
 
   /* Complete splitting input file into words on output.  */
 
   while (side->character != EOF)
     {
-      if (jump_trigger)
+      if (interrupted)
 	longjmp (signal_label, 1);
 
       skip_whitespace (side);
@@ -565,7 +611,6 @@ split_file_into_words (SIDE *side)
       copy_word (side, side->temp_file);
       putc ('\n', side->temp_file);
     }
-  fclose (side->file);
   fclose (side->temp_file);
 }
 
@@ -578,7 +623,7 @@ split_file_into_words (SIDE *side)
 | assumed to have the value of ARG2.  Return 0 if any error found.   |
 `-------------------------------------------------------------------*/
 
-int
+static int
 decode_directive_line (void)
 {
   int value;			/* last scanned value */
@@ -587,6 +632,7 @@ decode_directive_line (void)
 
   error = 0;
   state = 0;
+  value = 0;
   while (!error && state < 4)
     {
 
@@ -650,7 +696,7 @@ decode_directive_line (void)
 | Skip SIDE until some word ORDINAL, included.  |
 `----------------------------------------------*/
 
-void
+static void
 skip_until_ordinal (SIDE *side, int ordinal)
 {
   while (side->position < ordinal)
@@ -664,7 +710,7 @@ skip_until_ordinal (SIDE *side, int ordinal)
 | Copy SIDE until some word ORDINAL, included.  |
 `----------------------------------------------*/
 
-void
+static void
 copy_until_ordinal (SIDE *side, int ordinal)
 {
   while (side->position < ordinal)
@@ -678,29 +724,19 @@ copy_until_ordinal (SIDE *side, int ordinal)
 | Study diff output and use it to drive reformatting.  |
 `-----------------------------------------------------*/
 
-void
+static void
 reformat_diff_output (void)
 {
   int resync_left;		/* word position for left resynchronisation */
   int resync_right;		/* word position for rigth resynchronisation */
 
-  /* Open input files.  */
+  /* Rewind input files.  */
 
-  left_side->file = fopen (left_side->filename, "r");
-  if (left_side->file == NULL)
-    {
-      perror (left_side->filename);
-      exit (EXIT_OTHER_REASON);
-    }
+  rewind (left_side->file);
   left_side->character = getc (left_side->file);
   left_side->position = 0;
 
-  right_side->file = fopen (right_side->filename, "r");
-  if (right_side->file == NULL)
-    {
-      perror (right_side->filename);
-      exit (EXIT_OTHER_REASON);
-    }
+  rewind(right_side->file);
   right_side->character = getc (right_side->file);
   right_side->position = 0;
 
@@ -708,7 +744,7 @@ reformat_diff_output (void)
 
   while (1)
     {
-      if (jump_trigger)
+      if (interrupted)
 	longjmp (signal_label, 1);
 
       /* Skip any line irrelevant to this program.  */
@@ -851,18 +887,19 @@ reformat_diff_output (void)
 | Lauch the diff program.  |
 `-------------------------*/
 
-void
+static void
 launch_input_program (void)
 {
   /* Launch the diff program.  */
 
-  input_file = readpipe (DIFF_PROGRAM, left_side->temp_name,
-			 right_side->temp_name, NULL);
+  if (ignore_case)
+    input_file = readpipe (DIFF_PROGRAM, "-c", left_side->temp_name,
+			   right_side->temp_name, NULL);
+  else
+    input_file = readpipe (DIFF_PROGRAM, left_side->temp_name,
+			   right_side->temp_name, NULL);
   if (!input_file)
-    {
-      perror (DIFF_PROGRAM);
-      exit (EXIT_OTHER_REASON);
-    }
+    error (EXIT_OTHER_REASON, errno, DIFF_PROGRAM);
   character = getc (input_file);
 }
 
@@ -870,7 +907,7 @@ launch_input_program (void)
 | Complete the diff program.  |
 `----------------------------*/
 
-void
+static void
 complete_input_program (void)
 {
   fclose (input_file);
@@ -881,10 +918,11 @@ complete_input_program (void)
 | Launch the output pager if any.  |
 `---------------------------------*/
 
-void
+static void
 launch_output_program (void)
 {
   char *program;		/* name of the pager */
+  char *basename;		/* basename of the pager */
 
   /* Check if a output program should be called, and which one.  Avoid
      all paging if only statistics are needed.  */
@@ -908,7 +946,7 @@ launch_output_program (void)
   /* Ensure the termcap initialization string is sent to stdout right
      away, never to the pager.  */
 
-#ifdef HAVE_TPUTS
+#if HAVE_TPUTS
   if (termcap_init_string)
     {
       tputs (termcap_init_string, 0, putc_for_tputs);
@@ -922,14 +960,16 @@ launch_output_program (void)
     {
       output_file = writepipe (program, NULL);
       if (!output_file)
-	{
-	  perror (program);
-	  exit (EXIT_OTHER_REASON);
-	}
+	error (EXIT_OTHER_REASON, errno, program);
 
       /* If we are paging to less, use printer mode, not display mode.  */
 
-      if (strstr (program, "less"))
+      if (basename = strrchr ("less", '/'), basename)
+	basename++;
+      else
+	basename = program;
+
+      if (strstr (basename, "less"))
 	{
 	  find_termcap = 0;
 	  overstrike = 1;
@@ -942,7 +982,7 @@ launch_output_program (void)
 | Complete the pager program.  |
 `-----------------------------*/
 
-void
+static void
 complete_output_program (void)
 {
 
@@ -978,7 +1018,7 @@ complete_output_program (void)
   /* Ensure the termcap termination string is sent to stdout, never to
      the pager.  Moreover, the pager has terminated already.  */
 
-#ifdef HAVE_TPUTS
+#if HAVE_TPUTS
   if (termcap_end_string)
     {
       output_file = stdout;
@@ -991,7 +1031,7 @@ complete_output_program (void)
 | Print accumulated statistics.	 |
 `-------------------------------*/
 
-void
+static void
 print_statistics (void)
 {
   int count_common_left;	/* words unchanged in left file */
@@ -1059,30 +1099,36 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\
 `-----------------------------*/
 
 static void
-usage_and_exit (void)
+usage (int status)
 {
-  fprintf (stderr, "\
-Usage: %s [ OPTION ... ] OLD_FILE NEW_FILE\n\
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n", program_name);
+  else
+    {
+      printf ("\
+Usage: %s [OPTION]... FILE1 FILE2\n", program_name);
+      printf ("\
+Mandatory arguments to long options are mandatory for short options too.\n\
 \n\
   -C, --copyright            print Copyright then exit\n\
+  -V, --version              print program version then exit\n\
   -1, --no-deleted           inhibit output of deleted words\n\
   -2, --no-inserted          inhibit output of inserted words\n\
   -3, --no-common            inhibit output of common words\n\
   -a, --auto-pager           automatically calls a pager\n\
   -h, --help                 print this help\n\
+  -i, --ignore-case          fold character case while comparing\n\
   -l, --less-mode            variation of printer mode for \"less\"\n\
   -n, --avoid-wraps          do not extend fields through newlines\n\
   -p, --printer              overstrike as for printers\n\
   -s, --statistics           say how many words deleted, inserted etc.\n\
   -t, --terminal             use termcap as for terminal displays\n\
-  -v, --version              print program version then exit\n\
-  -w, --start-delete STRING  string to mark beginning of delete region\n\
-  -x, --end-delete STRING    string to mark end of delete region\n\
-  -y, --start-insert STRING  string to mark beginning of insert region\n\
-  -z, --end-insert STRING    string to mark end of insert region\n\
-", program_name);
-
-  exit (EXIT_OTHER_REASON);
+  -w, --start-delete=STRING  string to mark beginning of delete region\n\
+  -x, --end-delete=STRING    string to mark end of delete region\n\
+  -y, --start-insert=STRING  string to mark beginning of insert region\n\
+  -z, --end-insert=STRING    string to mark end of insert region\n");
+    }
+  exit (status);
 }
 
 /*---------------.
@@ -1090,7 +1136,7 @@ Usage: %s [ OPTION ... ] OLD_FILE NEW_FILE\n\
 `---------------*/
 
 void
-main (int argc, char *argv[])
+main (int argc, char *const argv[])
 {
   int option_char;		/* option character */
 
@@ -1101,6 +1147,8 @@ main (int argc, char *argv[])
   inhibit_left = 0;
   inhibit_right = 0;
   inhibit_common = 0;
+
+  ignore_case = 0;
   show_statistics = 0;
   no_wrapping = 0;
   autopager = 0;
@@ -1125,9 +1173,9 @@ main (int argc, char *argv[])
   count_changed_left = 0;
   count_changed_right = 0;
 
-  while ((option_char
-	  = getopt_long (argc, argv, "123Cahdlnpstvw:x:y:z:", longopts, NULL)
-	  ) != EOF)
+  while (option_char = getopt_long (argc, argv, "123CVahidlnpstw:x:y:z:",
+				    longopts, NULL),
+	 option_char != EOF)
     switch (option_char)
       {
       case '1':
@@ -1144,12 +1192,23 @@ main (int argc, char *argv[])
 
       case 'C':
 	print_copyright ();
-	exit (EXIT_OTHER_REASON);
+	exit (0);
+
+      case 'v':
+	printf ("GNU %s %s\n%s\n", PRODUCT, VERSION, copyright);
+	exit (0);
 
       case 'a':
 	autopager = 1;
 	break;
 
+      case 'h':
+	usage (0);
+
+      case 'i':
+	ignore_case = 1;
+	break;
+
       case 'l':
 	if (find_termcap < 0)
 	  find_termcap = 0;
@@ -1170,20 +1229,13 @@ main (int argc, char *argv[])
 	break;
 
       case 't':
+#if HAVE_TPUTS
 	if (find_termcap < 0)
-	  {
-#ifdef HAVE_TPUTS
-	    find_termcap = 1;
-#else
-	    fprintf (stderr, "Cannot use -t, termcap not available.\n");
-	    exit (EXIT_OTHER_REASON);
-#endif
-	  }
+	  find_termcap = 1;
 	break;
-
-      case 'v':
-	fprintf (stderr, "%s\n%s\n", version, copyright);
-	exit (EXIT_OTHER_REASON);
+#else
+	error (EXIT_OTHER_REASON, 0, "Cannot use -t, termcap not available.");
+#endif
 
       case 'w':
 	user_delete_start = optarg;
@@ -1201,13 +1253,12 @@ main (int argc, char *argv[])
 	user_insert_end = optarg;
 	break;
 
-      case 'h':
       default:
-	usage_and_exit ();
+	usage (EXIT_OTHER_REASON);
       }
 
   if (optind + 2 != argc)
-    usage_and_exit ();
+    usage (EXIT_OTHER_REASON);
 
   /* If find_termcap still undecided, consider it unset.  However, set if
      autopager is set while stdout is directed to a terminal, but this
@@ -1218,11 +1269,23 @@ main (int argc, char *argv[])
 
   /* Setup file names and signals, then do it all.  */
 
-  left_side->filename = argv[optind++];
-  right_side->filename = argv[optind++];
+  if (strcmp (argv[optind], "") == 0 || strcmp (argv[optind], "-") == 0)
+    left_side->filename = NULL;
+  else
+    left_side->filename = argv[optind];
+  optind++;
   *left_side->temp_name = '\0';
+
+  if (strcmp (argv[optind], "") == 0 || strcmp (argv[optind], "-") == 0)
+    right_side->filename = NULL;
+  else
+    right_side->filename = argv[optind];
+  optind++;
   *right_side->temp_name = '\0';
 
+  if (left_side->filename == NULL && right_side->filename == NULL)
+    error (EXIT_OTHER_REASON, 0, "Both files cannot be standard input.");
+
   setup_signals ();
   input_file = NULL;
   output_file = NULL;
@@ -1256,12 +1319,15 @@ main (int argc, char *argv[])
   if (output_file)
     complete_output_program ();
 
+  if (interrupted)
+    exit (EXIT_OTHER_REASON);
+
   if (show_statistics)
     print_statistics ();
 
   if (count_isolated_left || count_isolated_right
       || count_changed_left || count_changed_right)
     exit (EXIT_ANY_DIFFERENCE);
-  else
-    exit (EXIT_NO_DIFFERENCES);
+
+  exit (EXIT_NO_DIFFERENCES);
 }
